@@ -57,7 +57,7 @@ const FADE_IN_TIME = 500;
 const FADE_OUT_TIME = 350;
 
 const BLUR_BRIGHTNESS = 0.4;
-const BLUR_SIGMA = 20.0;
+const BLUR_RADIUS = 40.0;
 
 const OPEN_WHEN_IDLE_MIN_REMAINING_TIME = 3.0;
 
@@ -130,7 +130,7 @@ class PomodoroBlurredLightbox extends Lightbox {
         if (!this._background) {
             // Clone the group that contains all of UI on the screen. This is the
             // chrome, the windows, etc.
-            this._background = new Clutter.Clone({source: Main.uiGroup, clip_to_allocation: true});
+            this._background = new Clutter.Clone({source: Main.uiGroup, clip_to_allocation: false});
             this._background.set_background_color(BACKGROUND_COLOR);
             this._background.add_effect_with_name('blur', new Shell.BlurEffect());
             this.set_child(this._background);
@@ -154,7 +154,7 @@ class PomodoroBlurredLightbox extends Lightbox {
             if (effect) {
                 effect.set({
                     brightness: BLUR_BRIGHTNESS,
-                    sigma: BLUR_SIGMA * themeContext.scale_factor,
+                    radius: BLUR_RADIUS * themeContext.scale_factor,
                 });
                 effect.queue_repaint();
             }
@@ -226,7 +226,7 @@ class OverlayManager {
                 name: 'overlayGroup',
                 reactive: false,
             });
-            global.stage.add_actor(this._overlayGroup);
+            global.stage.add_child(this._overlayGroup);
             global.stage.set_child_above_sibling(this._overlayGroup, null);
         }
 
@@ -245,16 +245,13 @@ class OverlayManager {
         }
 
         for (const overlayData of this._overlayActors)
-            this._overlayGroup.add_actor(overlayData.actor);
+            this._overlayGroup.add_child(overlayData.actor);
     }
 
     _destroyOverlayGroup() {
         if (this._overlayGroup) {
-            for (const overlayData of this._overlayActors)
-                this._overlayGroup.remove_actor(overlayData.actor);
-
-
-            global.stage.remove_actor(this._overlayGroup);
+            this._overlayGroup.remove_all_children();
+            global.stage.remove_child(this._overlayGroup);
             this._overlayGroup = null;
         }
 
@@ -272,8 +269,8 @@ class OverlayManager {
         } else {
             chromeData.actor.ref();
             try {
-                Main.layoutManager.uiGroup.remove_actor(chromeData.actor);
-                global.stage.add_actor(chromeData.actor);
+                Main.layoutManager.uiGroup.remove_child(chromeData.actor);
+                global.stage.add_child(chromeData.actor);
             } finally {
                 chromeData.actor.unref();
             }
@@ -297,8 +294,8 @@ class OverlayManager {
         } else {
             chromeData.actor.ref();
             try {
-                global.stage.remove_actor(chromeData.actor);
-                Main.layoutManager.uiGroup.add_actor(chromeData.actor);
+                global.stage.remove_child(chromeData.actor);
+                Main.layoutManager.uiGroup.add_child(chromeData.actor);
             } finally {
                 chromeData.actor.unref();
             }
@@ -337,7 +334,6 @@ class OverlayManager {
 
         if (visibleCount > 0)
             this._raiseChrome();
-
         else
             this._lowerChrome();
     }
@@ -455,7 +451,7 @@ const ModalDialog = GObject.registerClass({
         // size requests.
         this._layout = new St.Widget({layout_manager: new Clutter.BinLayout()});
         this._layout.add_constraint(this._monitorConstraint);
-        this.add_actor(this._layout);
+        this.add_child(this._layout);
 
         // Lightbox will be a direct child of the ModalDialog
         this._lightbox = extension.pluginSettings.get_boolean('blur-effect')
@@ -574,14 +570,13 @@ const ModalDialog = GObject.registerClass({
     _onIdleMonitorBecameIdle(monitor) {  // eslint-disable-line no-unused-vars
         let pushModalTries = 0;
         const pushModalInterval = Math.floor(1000 / PUSH_MODAL_RATE);
-        const timestamp = global.get_current_time();
 
         if (this._pushModalWatchId) {
             this._idleMonitor.remove_watch(this._pushModalWatchId);
             this._pushModalWatchId = 0;
         }
 
-        if (this.pushModal(timestamp))
+        if (this.pushModal())
             return;
 
         this._pushModalSource = GLib.timeout_add(
@@ -748,7 +743,7 @@ const ModalDialog = GObject.registerClass({
     // Drop modal status without closing the dialog; this makes the
     // dialog insensitive as well, so it needs to be followed shortly
     // by either a close() or a pushModal()
-    popModal(timestamp) {
+    popModal() {
         this._disconnectPushModalSignals();
 
         if (this._keyFocusOutId) {
@@ -764,25 +759,21 @@ const ModalDialog = GObject.registerClass({
         if (!this._hasModal)
             return;
 
-        Main.popModal(this._grab, timestamp);
+        Main.popModal(this._grab);
         this._grab = null;
         this._hasModal = false;
 
         this._lightbox.reactive = false;
     }
 
-    pushModal(timestamp) {
+    pushModal() {
         if (this._hasModal)
             return true;
 
         if (this.state === DialogState.CLOSED || this.state === DialogState.CLOSING || this._destroyed)
             return false;
 
-        let params = {actionMode: Shell.ActionMode.SYSTEM_MODAL};
-        if (timestamp)
-            params['timestamp'] = timestamp;
-
-        let grab = Main.pushModal(this, params);
+        const grab = Main.pushModal(this, {actionMode: Shell.ActionMode.SYSTEM_MODAL});
         if (grab && grab.get_seat_state() !== Clutter.GrabState.ALL) {
             Utils.logWarning('Unable become fully modal');
             Main.popModal(grab);
@@ -950,9 +941,9 @@ class PomodoroEndDialog extends ModalDialog {
             style_class: 'extension-pomodoro-dialog-box',
             vertical: true,
         });
-        box.add_actor(this._timerLabel);
-        box.add_actor(this._descriptionLabel);
-        this._layout.add_actor(box);
+        box.add_child(this._timerLabel);
+        box.add_child(this._descriptionLabel);
+        this._layout.add_child(box);
 
         this._timerStateChangedId = this._timer.connect('state-changed', this._onTimerStateChanged.bind(this));
 
