@@ -181,7 +181,7 @@ const IndicatorMenu = class extends PopupMenu.PopupMenu {
                 else
                     this._onResumeClicked();
             });
-        buttonsBox.add_actor(pauseResumeButton);
+        buttonsBox.add_child(pauseResumeButton);
 
         const skipStopButton = this._createIconButton('gnome-pomodoro-stop-symbolic', _('Stop Timer'));
         skipStopButton.connect('clicked',
@@ -191,7 +191,7 @@ const IndicatorMenu = class extends PopupMenu.PopupMenu {
                 else
                     this._onStopClicked();
             });
-        buttonsBox.add_actor(skipStopButton);
+        buttonsBox.add_child(skipStopButton);
 
         const blinkingGroup = this._indicator.blinkingGroup;
         blinkingGroup.addActor(timerLabel);
@@ -639,18 +639,18 @@ const IconIndicator = class extends EventEmitter {
 
         /* background pie */
         if (isBreak || !isRunning) {
-            Clutter.cairo_set_source_color(cr, this._secondaryColor);
+            cr.setSourceColor(this._secondaryColor);
             cr.arcNegative(0, 0, radius, angle1, angle2);
             cr.setLineWidth(2.2 * scaleFactor);
             cr.stroke();
         } else {
-            Clutter.cairo_set_source_color(cr, this._secondaryColor);
+            cr.setSourceColor(this._secondaryColor);
             cr.arc(0, 0, radius, 0.0, 2.0 * Math.PI);
             cr.setLineWidth(2.2 * scaleFactor);
             cr.stroke();
 
             if (angle2 > angle1) {
-                Clutter.cairo_set_source_color(cr, this._primaryColor);
+                cr.setSourceColor(this._primaryColor);
                 cr.arcNegative(0, 0, radius, angle1, angle2);
                 cr.setOperator(Cairo.Operator.CLEAR);
                 cr.setLineWidth(3.5 * scaleFactor);
@@ -731,7 +731,7 @@ class PomodoroIndicator extends PanelMenu.Button {
             x_align: Clutter.ActorAlign.CENTER,
             y_align: Clutter.ActorAlign.CENTER,
         });
-        this.add_actor(this._iconBox);
+        this.add_child(this._iconBox);
 
         this._blinking = false;
         this._blinkingGroup = new Utils.TransitionGroup();
