@@ -46,29 +46,29 @@ class PomodoroScreenShieldWidget extends St.Widget {
             vertical: true,
             x_expand: true,
         });
-        this.add_actor(vbox);
+        this.add_child(vbox);
 
         const hbox = new St.BoxLayout();
-        vbox.add_actor(hbox);
+        vbox.add_child(hbox);
 
         const contentBox = new St.BoxLayout({
             style_class: 'extension-pomodoro-widget-content',
             vertical: true,
             x_expand: true,
         });
-        hbox.add_actor(contentBox);
+        hbox.add_child(contentBox);
 
         const blinkingGroup = new Utils.TransitionGroup();
 
         const titleLabel = new St.Label({style_class: 'extension-pomodoro-widget-title'});
-        contentBox.add_actor(titleLabel);
+        contentBox.add_child(titleLabel);
 
         const messageLabel = new St.Label({style_class: 'extension-pomodoro-widget-message', text: '15 minutes remaining'});
-        contentBox.add_actor(messageLabel);
+        contentBox.add_child(messageLabel);
         blinkingGroup.addActor(messageLabel);
 
         const buttonsBox = new St.BoxLayout();
-        hbox.add_actor(buttonsBox);
+        hbox.add_child(buttonsBox);
 
         const pauseResumeButton = this._createIconButton('gnome-pomodoro-pause-symbolic', _('Pause Timer'));
         pauseResumeButton.connect('clicked',
@@ -79,7 +79,7 @@ class PomodoroScreenShieldWidget extends St.Widget {
                 else
                     this._timer.resume();
             });
-        buttonsBox.add_actor(pauseResumeButton);
+        buttonsBox.add_child(pauseResumeButton);
         blinkingGroup.addActor(pauseResumeButton);
 
         const skipStopButton = this._createIconButton('gnome-pomodoro-stop-symbolic', _('Stop Timer'));
@@ -90,7 +90,7 @@ class PomodoroScreenShieldWidget extends St.Widget {
                 else
                     this._timer.stop();
             });
-        buttonsBox.add_actor(skipStopButton);
+        buttonsBox.add_child(skipStopButton);
 
         this._blinkingGroup = blinkingGroup;
         this._titleLabel = titleLabel;
@@ -361,9 +361,8 @@ export class ScreenShieldManager extends EventEmitter {
         super();
 
         this._timer = timer;
-        this._timerState = State.NULL;
+        this._timerState = null;
         this._widget = null;
-        this._previousTimerState = State.NULL;
         this._destroying = false;
 
         this._annoucementTimeoutId = 0;
@@ -371,7 +370,7 @@ export class ScreenShieldManager extends EventEmitter {
         this._timerPausedId = this._timer.connect('paused', this._onTimerPaused.bind(this));
         this._timerResumedId = this._timer.connect('resumed', this._onTimerResumed.bind(this));
 
-        this._onTimerStateChanged();
+        this._refresh();
     }
 
     get timer() {
@@ -442,16 +441,13 @@ export class ScreenShieldManager extends EventEmitter {
         }
     }
 
-    _onTimerStateChanged() {
+    _refresh() {
         const timerState = this._timer.getState();
         const isPaused = this._timer.isPaused();
 
-        if (timerState !== this._timerState)
-            Utils.wakeUpScreen();
-
         this._unscheduleAnnoucement();
 
-        if (!isPaused)
+        if (timerState !== State.NULL && !isPaused)
             this._scheduleAnnoucement();
 
         if (timerState !== State.NULL) {
@@ -462,6 +458,18 @@ export class ScreenShieldManager extends EventEmitter {
         }
     }
 
+    _onTimerStateChanged() {
+        const timerState = this._timer.getState();
+
+        // skip waking up if previous state was unknown
+        if (timerState !== this._timerState && this._timerState !== null)
+            Utils.wakeUpScreen();
+
+        this._refresh();
+
+        this._timerState = timerState;
+    }
+
     _onTimerPaused() {
         this._onTimerStateChanged();
     }
