@@ -28,7 +28,7 @@ import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
 import * as Params from 'resource:///org/gnome/shell/misc/params.js';
 
 import {PomodoroClient} from './dbus.js';
-import {IssueNotification} from './notifications.js';
+import {IssueNotification, getDefaultSource} from './notifications.js';
 import * as Config from './config.js';
 import * as Utils from './utils.js';
 
@@ -248,8 +248,9 @@ export class Timer extends EventEmitter {
     }
 
     _notifyServiceNotInstalled() {
+        const source = new getDefaultSource();
         const notification = new IssueNotification(_('Failed to run <i>%s</i> service').format(Config.PACKAGE_NAME));
-        notification.show();
+        source.addNotification(notification);
     }
 
     destroy() {
@@ -402,19 +403,19 @@ class PomodoroTimerLabel extends St.BoxLayout {
             text: '0',
             text_align: Pango.Alignment.RIGHT,
         });
-        this.add_actor(this._minutesLabel);
+        this.add_child(this._minutesLabel);
 
         this._separatorLabel = new St.Label({
             text: ':',
         });
         this._separatorLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
-        this.add_actor(this._separatorLabel);
+        this.add_child(this._separatorLabel);
 
         this._secondsLabel = new MonospaceLabel({
             text: '00',
             text_align: Pango.Alignment.LEFT,
         });
-        this.add_actor(this._secondsLabel);
+        this.add_child(this._secondsLabel);
 
         this.connect('destroy', this._onDestroy.bind(this));
     }
