@@ -18,12 +18,13 @@
  *
  */
 
-const { Gio, Meta, Shell } = imports.gi;
+import Meta from 'gi://Meta';
+import Shell from 'gi://Shell';
 
-const Main = imports.ui.main;
-const Signals = imports.misc.signals;
+import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
 
-const Extension = imports.misc.extensionUtils.getCurrentExtension();
+import * as Config from './config.js';
 
 const ENABLED_EXTENSIONS_KEY = 'enabled-extensions';
 
@@ -36,7 +37,7 @@ const VIDEO_PLAYER_CATEGORIES = [
 ];
 
 
-var Patch = class extends Signals.EventEmitter {
+export const Patch = class extends EventEmitter {
     constructor(object, overrides) {
         super();
 
@@ -48,17 +49,15 @@ var Patch = class extends Signals.EventEmitter {
         for (let name in this.overrides) {
             this.initial[name] = this.object[name];
 
-            if (typeof(this.initial[name]) == 'undefined') {
+            if (typeof this.initial[name] == 'undefined')
                 logWarning(`Property "${name}" for ${this.object} is not defined`);
-            }
         }
     }
 
     apply() {
         if (!this.applied) {
-            for (let name in this.overrides) {
+            for (let name in this.overrides)
                 this.object[name] = this.overrides[name];
-            }
 
             this.applied = true;
 
@@ -68,9 +67,8 @@ var Patch = class extends Signals.EventEmitter {
 
     revert() {
         if (this.applied) {
-            for (let name in this.overrides) {
+            for (let name in this.overrides)
                 this.object[name] = this.initial[name];
-            }
 
             this.applied = false;
 
@@ -85,8 +83,7 @@ var Patch = class extends Signals.EventEmitter {
 };
 
 
-var TransitionGroup = class {
-
+export const TransitionGroup = class {
     /* Helper class to share property transition between multiple actors */
 
     constructor() {
@@ -100,9 +97,8 @@ var TransitionGroup = class {
 
     _findActor(actor) {
         for (var index = 0; index < this._actors.length; index++) {
-            if (this._actors[index].actor === actor) {
+            if (this._actors[index].actor === actor)
                 return index;
-            }
         }
 
         return -1;
@@ -111,12 +107,11 @@ var TransitionGroup = class {
     addActor(actor) {
         let index = this._findActor(actor);
 
-        if (!actor || index >= 0) {
+        if (!actor || index >= 0)
             return;
-        }
 
         const meta = {
-            actor: actor,
+            actor,
             destroyId: actor.connect('destroy', () => {
                 this.removeActor(actor);
                 meta.destroyId = 0;
@@ -124,39 +119,36 @@ var TransitionGroup = class {
         };
         this._actors.push(meta);
 
-        if (!this._referenceActor) {
+        if (!this._referenceActor)
             this._setReferenceActor(actor);
-        }
     }
 
     removeActor(actor) {
         let index = this._findActor(actor);
         if (index >= 0) {
             const meta = this._actors.splice(index, 1);
-            if (meta.destroyId) {
+            if (meta.destroyId)
                 actor.disconnect(meta.destroyId);
-            }
         }
 
-        if (this._referenceActor === actor) {
+        if (this._referenceActor === actor)
             this._setReferenceActor(this._actors.length > 0 ? this._actors[0].actor : null);
-        }
     }
 
     easeProperty(name, target, params) {
         let onStopped = params.onStopped;
         let onComplete = params.onComplete;
 
-        this._actors.forEach((meta) => {
+        this._actors.forEach(meta => {
             let localParams = Object.assign({
-                onStopped: (isFinished) => {
+                onStopped: isFinished => {
                     if (onStopped && meta.actor === this._referenceActor)
                         onStopped(isFinished);
                 },
                 onComplete: () => {
-                     if (onComplete && meta.actor === this._referenceActor)
-                         onComplete();
-                }
+                    if (onComplete && meta.actor === this._referenceActor)
+                        onComplete();
+                },
             }, params);
 
             meta.actor.ease_property(name, target, localParams);
@@ -165,77 +157,82 @@ var TransitionGroup = class {
 
     setProperty(name, target) {
         let properties = {};
-        properties[name] = target
+        properties[name] = target;
 
-        this._actors.forEach((meta) => {
+        this._actors.forEach(meta => {
             meta.actor.set(properties);
         });
     }
 
     removeAllTransitions() {
-        this._actors.forEach((meta) => {
+        this._actors.forEach(meta => {
             meta.actor.remove_all_transitions();
         });
     }
 
     destroy() {
-        this._actors.slice().forEach((meta) => {
+        this._actors.slice().forEach(meta => {
             this.removeActor(meta.actor);
         });
     }
-}
+};
 
 
+/**
+ *
+ * @param {Array} subset - subset
+ * @param {Array} set - set
+ */
 function isSubset(subset, set) {
     for (let value of subset) {
-        if (set.indexOf(value) < 0) {
+        if (set.indexOf(value) < 0)
             return false;
-        }
     }
 
     return true;
 }
 
 
+/**
+ *
+ * @param {object} app - app object
+ */
 function _isVideoPlayer(app) {
     const appInfo = app.get_app_info();
-    if (!appInfo) {
+    if (!appInfo)
         return false;
-    }
 
     const categoriesStr = appInfo.get_categories();
     const categories    = categoriesStr ? categoriesStr.split(';') : [];
 
-    if (!categories.length) {
+    if (!categories.length)
         return false;
-    }
 
     for (let videoPlayerCategories of VIDEO_PLAYER_CATEGORIES) {
-        if (isSubset(videoPlayerCategories, categories)) {
+        if (isSubset(videoPlayerCategories, categories))
             return true;
-        }
     }
 
     return false;
 }
 
 
-function isVideoPlayerOpen() {
+/**
+ *
+ */
+export function isVideoPlayerOpen() {
     const apps = Shell.AppSystem.get_default().get_running();
 
     for (let app of apps) {
-        if (!_isVideoPlayer(app)) {
+        if (!_isVideoPlayer(app))
             continue;
-        }
 
         for (let window of app.get_windows()) {
-            if (window.window_type !== Meta.WindowType.NORMAL || window.is_hidden()) {
+            if (window.window_type !== Meta.WindowType.NORMAL || window.is_hidden())
                 continue;
-            }
 
-            if (window.fullscreen) {
+            if (window.fullscreen)
                 return true;
-            }
         }
     }
 
@@ -243,52 +240,67 @@ function isVideoPlayerOpen() {
 }
 
 
-function logError(error) {
-    Main.extensionManager.logExtensionError(Extension.metadata.uuid, error);
+/**
+ *
+ * @param {Error} error - error
+ */
+export function logError(error) {
+    Main.extensionManager.logExtensionError(Config.EXTENSION_UUID, error);
 }
 
 
-function logWarning(message) {
+/**
+ *
+ * @param {string} message - error
+ */
+export function logWarning(message) {
     console.warn(`Pomodoro: ${message}`);
 }
 
 
-function versionCheck(required) {
-    let current = imports.misc.config.PACKAGE_VERSION;
-    let currentArray = current.split('.');
-    let requiredArray = required.split('.');
+/**
+ *
+ * @param {string} required - version required
+ */
+export function isVersionAtLeast(required) {
+    const current = Config.PACKAGE_VERSION;
+    const currentArray = current.split('.');
+    const requiredArray = required.split('.');
 
     if (requiredArray[0] <= currentArray[0] &&
         requiredArray[1] <= currentArray[1] &&
-        (requiredArray[2] <= currentArray[2] ||
-         requiredArray[2] == undefined)) {
+        (requiredArray[2] <= currentArray[2] || requiredArray[2] === undefined))
         return true;
-    }
 
     return false;
 }
 
 
-function disableExtension(uuid) {
-    let enabledExtensions = global.settings.get_strv(ENABLED_EXTENSIONS_KEY);
-    let extensionIndex = enabledExtensions.indexOf(uuid);
+/**
+ *
+ * @param {string} uuid - extension uuid
+ */
+export function disableExtension(uuid) {
+    const enabledExtensions = global.settings.get_strv(ENABLED_EXTENSIONS_KEY);
+    const extensionIndex = enabledExtensions.indexOf(uuid);
 
-    if (extensionIndex != -1) {
+    if (extensionIndex >= 0) {
         enabledExtensions.splice(extensionIndex, 1);
         global.settings.set_strv(ENABLED_EXTENSIONS_KEY, enabledExtensions);
     }
 }
 
 
-function wakeUpScreen() {
+/**
+ *
+ */
+export function wakeUpScreen() {
     if (Main.screenShield._dialog) {
         Main.screenShield._dialog.emit('wake-up-screen');
-    }
-    else {
+    } else {
         try {
             Main.screenShield._wakeUpScreen();
-        }
-        catch (error) {
+        } catch (error) {
             logWarning(`Error while waking up the screen: ${error}`);
         }
     }
